//! bare2.rs
//! Simple bare metal application
#![feature(used)]
#![no_std]

extern crate cortex_m;
extern crate cortex_m_rt;

#[macro_use]
extern crate cortex_m_debug;

// burns CPU cycles by just looping `i` times
fn wait(i: u32) {
    for _ in 0..i {
        // no operation (ensured not optimized out)
        cortex_m::asm::nop();
    }
}

fn main() {
    ipln!("Start");

    // a "mutable" (changeable) variable
    let mut s = 0;

    loop {
        ipln!("s = {}", s);
        s += 1;
        wait(100_000);
    }
}

// 1. build and run the application (debug build)
// start ITM tracing to console *version 0.1.1*
// > itmdump /tmp/itm.log
// or alternatively *version 0.2.0*
// > mkfifo /tmp/itm.log
// > itmdump -f /tmp/itm.log -F
//
// start openocd (in my case...)
// > openocd -f interface/stlink.cfg -f target/stm32f4x.cfg
//
// what is the output in the ITM console
// 
// Start
// s = 0
// s = 1
// s = 2
// ...
// 
// ** your answer here **
//
// estimate the frequency of the output, give freq in hz
//
// .8 hz
// 49 tick in 60s
//
// ** your answer here **
//
// commit your answers (bare2_1)
//
// 2. rebuild and run in release mode
// estimate the frequency of the output, give freq in hz
// ** your answer here **
//
// estimate the ratio between debug/release optimized code
// (speedup)
// ** your answer here **
//
// commit your answers (bare2_2)
//
// 3. optional
// inspect the generated binaries, and try stepping through the code
// for both debug and release binaries. How do they differ
// ** your answer here **
//
// commit your answers (bare2_3)

// As we are not using interrupts, we just register a dummy catch all handler
#[link_section = ".vector_table.interrupts"]
#[used]
static INTERRUPTS: [extern "C" fn(); 240] = [default_handler; 240];

extern "C" fn default_handler() {
    cortex_m::asm::bkpt();
}
